use actix_web::{
    get,
    http::StatusCode,
    put,
    web::{scope, Data, Json, ServiceConfig},
    HttpResponse, Responder,
};
use diesel::{insert_into, ExpressionMethods, QueryDsl, RunQueryDsl};

use crate::schema::posts::dsl::*;
use crate::schema::users::dsl::{id as user_id, users};
use crate::utils::response::failure;
use crate::State;
use crate::{
    database::models::post::{NewPost, Post},
    database::models::user::User,
    utils::jwt::AuthorizedUser,
};

#[get("")]
async fn get(state: Data<State>) -> impl Responder {
    let mut conn = match state.db.get() {
        Ok(conn) => conn,
        Err(why) => {
            return HttpResponse::InternalServerError()
                .body(format!("database connection error: {}", why))
        }
    };

    match posts.load::<Post>(&mut conn) {
        Ok(data) => HttpResponse::Ok().json(data),
        Err(why) => HttpResponse::InternalServerError().body(why.to_string()),
    }
}

#[put("")]
async fn create_post(
    state: Data<State>,
    auth: AuthorizedUser,
    mut post: Json<NewPost>,
) -> impl Responder {
    let mut conn = match state.db.get() {
        Ok(conn) => conn,
        Err(why) => {
            return HttpResponse::InternalServerError()
                .body(format!("database connection error: {}", why))
        }
    };

    let user = users
        .filter(user_id.eq(auth.user_id))
        .first::<User>(&mut conn);

    if user.is_err() {
        return HttpResponse::InternalServerError()
            .json(failure("cannot retrieve user from JWT token. sorry."));
    } else {
        post.author_id = user.unwrap().id;
    }

    match insert_into(posts).values(post.0).execute(&mut conn) {
        Ok(_) => HttpResponse::new(StatusCode::NO_CONTENT),
        Err(why) => HttpResponse::InternalServerError().json(failure(why.to_string())),
    }
}

pub fn init_routes(cfg: &mut ServiceConfig) {
    cfg.service(scope("/posts").service(get).service(create_post));
}
